library(eda)
express <- function(x,method=c("ravglogpm","avglogpm","li.wong","avdiff"),
                    normalize=T,
                    normalize.method=c("quantile","loess","contrasts"),
                    span=2/3,choose.subset=T,subset.size=5000,
                    verbose=T,maxit=1){
  if(!is.element(method,c("ravglogpm","avglogpm","li.wong","avdiff")))
    stop(paste(method,"not a method!\n"))
  if(verbose & normalize) cat("Performing",normalize.method[1],"normalization.\n")
  if(normalize) x <- normalize(x,method=normalize.method,span=span,choose.subset=choose.subset,subset.size=subset.size,verbose=verbose,maxit=maxit)
  if(verbose) cat("Computing",method[1],"expression\n")
  if(method=="ravglogpm"){
    if(verbose) cat("This can take various minutes\n")
    bg <- apply(log2(x$mm),2,function(x){
      aux <- density(x,kernel="epanechnikov",n=2^16)
      2^aux$x[order(-aux$y)[1]]})
    aux <- sweep(x$pm,2,bg)
    MIN <- min(aux[aux > 0])/2 ##this is going to be 0 expression
    aux[aux < 0] <- MIN
    aux <- log2(aux)
    geneids <- unique(x$id)
    ngenes <- length(geneids)
    e <- matrix(0,ngenes,x$nchips)
    if(verbose) cat("Computing median polish for",ngenes,"probe sets.")
    for(i in 1:ngenes){
      if(verbose){     if(i%%100==0) cat(".")}
      xx <- aux[x$id%in%geneids[i],]
      mp <- medpolish(xx,trace.iter=FALSE)
      e[i,] <- 2^(mp$overall + mp$col)
    }
    if(verbose) cat("\n")
    e[e<=MIN] <- 0
    dimnames(e) <- list(x$names[geneids],x$chip.names)
    return(e)
  } 
  
  if(method=="avglogpm"){
    bg <- apply(log2(x$mm),2,function(x){
      aux <- density(x,kernel="epanechnikov",n=2^16)
      2^aux$x[order(-aux$y)[1]]})
    aux <- sweep(x$pm,2,bg)
    aux[aux<=0] <- min(aux[aux>0])/2
    aux <- log2(aux)
    aux <- 2^apply(aux,2,function(y) tapply(y,x$id,mean))
    dimnames(aux) <- list(x$names,x$chip.names)
    return(aux)
  }
  if(method=="li.wong"){
    aux <- li.wong(x)$theta
    dimnames(aux) <- list(x$names,x$chip.names)
    return(aux)
  }
  if(method=="avdiff") return(avdiff(x))
}
