li.wong <- function(x,subset=NULL,ignore.mm=F,remove.outliers=T,
                    normal.array.quantile=0.5,
                    normal.resid.quantile=0.9,
                    large.threshold=3,
                    large.variation=0.8,
                    outlier.fraction=0.14,
                    delta = 1e-06,maxit=50,outer.maxit=50,verbose=F){
  if(missing(x)) stop("Argument x missing, with no default\n")
  if(ignore.mm)
    aux <- x$pm
  else
    aux <- x$pm-x$mm
  unique.probe.names <- x$names
  if(!is.null(subset)) unique.probe.names <- unique.probe.names[subset]
  n <- x$n
  theta <- matrix(0,n,x$nchips)
  sigma.theta <- matrix(0,n,x$nchips)
  outlier <- matrix(T,n,x$nchips)
  sigma.eps <- rep(0,n)
  convergence1 <- rep(T,n)
  convergence2 <- rep(T,n)
  for(i in 1:n){ 
    index <- x$id==i
    res <- fit.li.wong(t(aux[index,]),remove.outliers,normal.array.quantile,normal.resid.quantile,large.threshold,large.variation,outlier.fraction,delta,maxit,outer.maxit,verbose)
    theta[i,] <- res$theta
    sigma.theta[i,] <- res$sigma.theta
    sigma.eps[i] <- res$sigma.eps
    outlier[i,] <- res$theta.outliers
    convergence1[i] <- res$convergence1
    convergence2[i] <- res$convergence2
  }
  return(list(names=unique.probe.names,theta=theta,sigma.theta=sigma.theta,sigma.eps=sigma.eps,outlier=outlier,convergence1=convergence1,convergence2=convergence2))
}

