projvec <- function(vec1,vec2){
  sum(vec1*vec2)/sum(vec2^2) * vec2
}

reorder.vec <- function(vec1,ordering){
  vec1[ordering]
}

normalize.quantiles.robust <- function(x,outlier.const=1.5){
  n.chips <- dim(x)[2]
  n.probes <- dim(x)[1]
  
  x.sorted <- apply(x,2,sort)
  diag.vec <- rep(1/sqrt(n.chips),n.chips)

  #weed out outliers
  
  outlierdef <- outlier.const*(apply(x,2,quantile,prob=0.75) - apply(x,2,quantile,prob=0.25)) +  (apply(x,2,quantile,prob=0.75))
  outlier.inds <- matrix(0,n.probes,n.chips)
  for (i in 1:n.chips){
    outlier.inds[,i] <- x.sorted[,i] > outlierdef[i]
  }

  # do standard procedure where there is no outliers

  not.outlier <- rep(1,dim(x)[1])

  for (ind in 1:n.chips){
    not.outlier <- not.outlier & !outlier.inds[,ind]
  }
  
  norm.data <- matrix(0,n.probes,n.chips)
  norm.data[not.outlier,] <- t(apply(x.sorted[not.outlier,],1,projvec,diag.vec))

  if(sum(!not.outlier)){

       #   normalize non outliers to max of non outliers in dist where outlier

    chip.max <- NULL
  
    for (i in 1:n.chips){
      chip.max <- c(chip.max,max(x.sorted[!outlier.inds[,i],i]))   
    }

    outlier.rows <- (n.probes - 0:(sum(!not.outlier)-1))
    # deal first with the non outliers in outlier rows
    tmp.vec <- x.sorted[outlier.rows,]
    inds.vec <- 1:n.chips
    out.inds <- t(apply(matrix(outlier.inds[outlier.rows,],sum(!not.outlier),n.chips),1,"*",inds.vec))
    tmp.vec[!!out.inds] <- t(matrix(chip.max,n.chips,sum(!not.outlier)))[!!out.inds]
    tmp.vec <- t(apply(matrix(tmp.vec,sum(!not.outlier),n.chips),1,projvec,diag.vec))
    for (chips in 1:n.chips){
      norm.data[outlier.rows[!out.inds[,chips]],chips] <- tmp.vec[!out.inds[,chips],chips]
    }
    
    IQRS <- apply(x,2,quantile,prob=0.75) - apply(x,2,quantile,prob=0.25)
    Means <- apply(x,2,mean)
    for (outlier.rows in (n.probes - 0:(sum(!not.outlier)-1))){
      # deal with the outliers by scaling with pseudo outliers
      tmp.vec <- x.sorted[outlier.rows,]
      num.outliers <- sum(outlier.inds[outlier.rows,])
      out.inds <- (outlier.inds[outlier.rows,])*(1:n.chips)
      
      #createpseudo outlier
      for (i in out.inds){
        if (i){
          tmp.vec <- x.sorted[outlier.rows,]
          Outlier.const <- (tmp.vec[i] - Means[i])/IQRS[i]
          tmp.vec[-i] <- Means[-i] + IQRS[-i]*Outlier.const
          tmp.vec <- projvec(tmp.vec,diag.vec)
          norm.data[outlier.rows,i] <- tmp.vec[i]
        }
      
      }
    }
  }
  original.inds <- apply(x,2,rank)
  
  for (ind in 1:n.chips){
    norm.data[,ind] <- reorder.vec(norm.data[,ind],original.inds[,ind])
  }
  norm.data
}


qnrm <- function(x){
  dim1 <- dim(x$pm)
  datamat <- rbind(x$pm,x$mm)  
  norm.data <- normalize.quantiles.robust(datamat)
  output <- x
  output$pm <- norm.data[1:dim1[1],]
  output$mm <- norm.data[(1+dim1[1]):(2*dim1[1]),]
  output
}
