\name{read.cdffile}
\alias{read.cdffile}
\title{Read a CDF file data}
\description{
  Read the data contained in a CDF file.
}
\usage{
read.cdffile(file, compress=TRUE)
}
\arguments{
  \item{file}{the name of the CDF file}
  \item{compress}{whether the file is compressed or not...}
}
\value{
  The object return is a \code{\link{CDF}} object. It is a list, which
  contains the following items:
  \item{name}{a matrix of indices for names corresponding to each spot}
  \item{name.levels}{the names corresponding to the indices}
  \item{pbase}{the PBASE attribute in a CDF file. This is a matrix of PBASE
    indices.}
  \item{pbase.levels}{the PBASEs corresponding to the indices}
  \item{pbase}{the PTBASE thing in a CDF file. This is a matrix of TBASE
    indices.}
  \item{pbase.levels}{the TBASEs corresponding to the indices}
}
\details{
  In order to save memory, the name corresponding to each value in the
  CEL is a factor. As in R \code{factor} objects
  cannot be also of type matrix, the names corresponding to the indeces
  were stored in a vector of type \code{character} called
  \code{name.levels}. The same thing was done with the pbase and cbase
  information. 
}
\author{Laurent Gautier}
\seealso{
  \code{\link{CDF}}
  }
\examples{
data(CDF.example)
## to find where found all the probes corresponding to given name are found
theName <- "A28102_at"
index <- which(CDF.example$name.levels == theName)
theCoords <- which(CDF.example$name == index, arr.ind=TRUE)
## to have a visual representation of them
dimChip <- dim(CDF.example$name)
plot(c(1,dimChip[1]), c(1,dimChip[2]), type="n", xlab="", ylab="")
points(theCoords, col="red", pch=16)
}
\keyword{file}

